from enum import Enum
import openpyxl
import yaml

class ValueType(Enum):
    INCREMENT = 1
    STRING = 2
    LIST = 3

class Alignment(Enum):
    LEFT = 1
    CENTER = 2
    RIGHT = 3

    def excel_alignment(self):
        if self == Alignment.LEFT:
            return openpyxl.styles.alignment.Alignment(
                wrap_text=True, vertical='top', horizontal='left')
        if self == Alignment.CENTER:
            return openpyxl.styles.alignment.Alignment(
                wrap_text=True, vertical='top', horizontal='center')
        if self == Alignment.RIGHT:
            return openpyxl.styles.alignment.Alignment(
                wrap_text=True, vertical='top', horizontal='right')

class ColumnCondition:
    def __init__(
            self,
            value_type: ValueType | str = ValueType.STRING,
            width: int = None,
            alignment: Alignment | str = Alignment.LEFT
    ):
        self.type = value_type
        self.width = width
        if self.type == ValueType.INCREMENT:
            self.alignment = Alignment.RIGHT
        else:
            self.alignment = alignment


class ColumnConfig:
    def __init__(self):
        self.prepend_columns = {}
        self.conditions = {}
        self.append_columns = {}

    def all_conditions(self):
        return {**self.prepend_columns, **self.conditions, **self.append_columns}

    def list_columns(self):
        return [k for k, v in self.conditions.items() if v.type == ValueType.LIST]

    def increment_columns(self):
        return [k for k, v in self.conditions.items() if v.type == ValueType.INCREMENT]

    def parse(self, path: str):
        with open(path, 'r') as f:
            config = yaml.load(f, Loader=yaml.SafeLoader)

        if 'prepend' in config:
            for name, column in config['prepend'].items():
                self.prepend_columns[name] = self.parse_condition(column)

        if 'column_conditions' in config:
            for name, column in config['column_conditions'].items():
                self.conditions[name] = self.parse_condition(column)

        if 'append' in config:
            for name, column in config['append'].items():
                self.prepend_columns[name] = self.parse_condition(column)

    @staticmethod
    def parse_condition(condition: dict):
        """
        Parse a column condition from a dict

        :param condition:
        :return:

        Example:
        {
            'value_type': 'increment',
            'width': 10,
            'alignment': 'right'
        }

        >>> c = ColumnConfig.parse_condition({
        ...     'value_type': 'increment',
        ...     'width': 10,
        ...     'alignment': 'right'
        ... })
        >>> c.type
        <ValueType.INCREMENT: 1>
        >>> c.width
        10
        >>> ColumnConfig.parse_condition({
        ...     'value_type': 'increment',
        ...     'width': 10,
        ...     'alignment': 'right'
        ... }).alignment
        <Alignment.RIGHT: 3>
        """
        return ColumnCondition(
            ValueType[condition['value_type'].upper()] if 'value_type' in condition else ValueType.STRING,
            condition['width'] if 'width' in condition else None,
            Alignment[condition['alignment'].upper()] if 'alignment' in condition else Alignment.LEFT
        )



from setuptools import setup, find_packages
from codecs import open
import os


root_dir = os.path.abspath(os.path.dirname(__file__))
# def _requirements():
#     requirements_file_path = os.path.join(root_dir, 'requirements.txt')
#     return [name.rstrip() for name in open(requirements_file_path).readlines()]
#     install_requires = _requirements()

# Get the long description from the README file
readme_file_path = os.path.join(root_dir, 'README.rst')
with open(readme_file_path, encoding='utf-8') as f:
    long_description = f.read()

setup(
    name='mael',
    packages=find_packages(),
    # package_dir={'': 'mael'},
    # packages=find_packages(where='mael'),
    version='0.0.3.14',
    py_modules=['mael', 'mael.main', 'mael.initializer', 'mael.excel_builder', 'mael.column_config'],
    install_requires=[
        'pyyaml',
        'openpyxl'
    ],
    entry_points={
        'console_scripts': ['mael=mael.main:main'],
    },
    license='MIT',
    author='Kenji Otsuka',
    author_email='kok.fdcm@gmail.com',
    description='A tool to convert markdown file to excel.',
    long_description=long_description,
    url='https://github.com/KenjiOhtsuka/mael',
    include_package_data=True,
    # entry_points='''
    #     [console_scripts]
    #     myproject=myproject:cli
    # ''',
)

